/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


#include "../config.h"
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sockio.h>
#include <pvfs.h>

int parse_args(int argc, char **argv);
int usage(int argc, char **argv);

extern char *optarg;
extern int optind;

static char host[256] = "localhost";
static char fs[256] ="/";
static int port_nr = MGR_REQ_PORT;

int iod_ping(char *host, int port);

int main(int argc, char **argv) {
	int i, s, err, iod_err;
	struct mreq req;
	struct mack ack;
	char *data;
	struct iod_info *info;
	
	s = new_sock();

	parse_args(argc, argv);
	if (optind < argc) {
		usage(argc, argv);
	}

	req.majik_nr = MGR_MAJIK_NR;
	req.type = MGR_IOD_INFO;
	req.dsize = strlen(fs) + 1;

	err = connect_sock(s, host, port_nr);
	if (err < 0) goto mgr_oops;

	err = bsend(s, &req, sizeof(req));
	if (err < 0) goto mgr_oops;

	err = bsend(s, fs, req.dsize);
	if (err < 0) goto mgr_oops;

	err = brecv(s, &ack, sizeof(ack));
	if (err < 0) goto mgr_oops;

	if (ack.status != 0) {
		printf("error: mgr (%s:%d) is alive, but %s is not recognized as a file system.\n", 
			host, port_nr, fs);
		printf("pvfs file system %s has issues.\n", fs);
		close(s);
		return 1;
	}

	/* make room for data, receive */
	data = malloc(ack.dsize);
	err = brecv(s, data, ack.dsize);
	if (err < 0) {
		printf("error: mgr (%s:%d) is alive but error occurred receiving iod info for %s.\n", 
			host, port_nr, fs);
		printf("pvfs file system %s has issues.\n", fs);
		close(s);
		return 1;
	}

	printf("mgr (%s:%d) is responding.\n", host, port_nr);

	info = (struct iod_info *) data;
	iod_err = 0;
	for (i=0; i < ack.ack.iod_info.nr_iods; i++) {
		static char iod_host[]="xxx.xxx.xxx.xxx\0";
		unsigned char *uc = (unsigned char *)&(info[i].addr.sin_addr);
		int iod_port;

		iod_port = ntohs(info[i].addr.sin_port);
		sprintf(iod_host, "%d.%d.%d.%d", uc[0], uc[1], uc[2], uc[3]);

		err = iod_ping(iod_host, iod_port);
		if (err < 0) {
			printf("iod %d (%s:%d) is down.\n", i, iod_host,
				iod_port);
			iod_err++;
		}
		else {
			printf("iod %d (%s:%d) is responding.\n", i,
				iod_host, iod_port);
		}
	}

	if (iod_err) {
		printf("pvfs file system %s has issues.\n", fs);
		return 1;
	}
	else {
		printf("pvfs file system %s is fully operational.\n", fs);
		return 0;
	}

mgr_oops:
	close(s);
	printf("mgr (%s:%d) is down.\n", host, port_nr);
	return 1;
}


int parse_args(int argc, char **argv)
{
	int c;

	while ((c = getopt(argc, argv, "h:p:f:")) != EOF) {
		switch(c) {
			case 'h':
				strncpy(host, optarg, 255);
				host[255] = 0;
				break;
			case 'p':
				port_nr = atoi(optarg);
				break;
			case 'f':
				strncpy(fs, optarg, 255);
				break;
			case '?':
				usage(argc, argv);
				break;
		}
	}
	return 1;
} /* end of PARSE_ARGS() */


/* iod_ping(host, port) - does a blocking ping to an iod
 *
 * Returns 0 on success, -1 on failure.
 */
int iod_ping(char *host, int port)
{
	int s, err;
	struct ireq req;
	struct iack ack;
	
	req.majik_nr = IOD_MAJIK_NR;
	req.type = IOD_NOOP;
	req.dsize = 0;

	s = new_sock();

	err = connect_sock(s, host, port);
	if (err < 0) goto oops;

	err = bsend(s, &req, sizeof(req));
	if (err < 0) goto oops;

	err = brecv(s, &ack, sizeof(ack));
	if (err < 0) goto oops;

	close(s);
	return 0;

oops:
	close(s);
	return -1;
}

int usage(int argc, char **argv)
{
	fprintf(stderr, "Usage: pvfs-ping [-h host] [-p port] [-f metadata_dir]\n");
	exit(1);
} /* end of USAGE() */

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 */
